{*******************************************************************************
* Unit      : Armour                                                           *
********************************************************************************
* Purpose   : Defines the armour object                                        *
********************************************************************************
* Copyright : This module is copyright TSM Inc. (Ian Sparkes) 1998.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 28.02.98  - 1.0 Original module                                  *
********************************************************************************
* Switches  : none                                                             *
*******************************************************************************}
unit Armour;

interface

const
     LINELENGTH = 64;

{*******************************************************************************
* Type      : Tbase64                                                          *
********************************************************************************
* Purpose   : provides support for XX-code armour                              *
*******************************************************************************}
type TBase64 = class
  public
     constructor Create;
     function  StringToBase64(StringToEncode: string): string;
     function  Base64ToString(StringToDecode: string): string;
  private
     Counter:  integer;
     CharCounter: integer;
     ReturnString: string;
     last: byte;
     this: byte;
     procedure Init;
     procedure ByteToBase64(b: byte);
     procedure Base64ToHex(c: byte);
     procedure Base64ToByte(c: byte);
     procedure Base64Final;
     procedure ByteFinal;
end;

implementation

uses
     SysUtils;

const
     BinToAsc : Array [0..63] of Char = ('+', '-','0','1','2','3','4','5','6','7',
                                         '8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
                                         'P','Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d',
                                         'e','f','g','h','i','j','k','l','m','n','o','p','q','r','s',
                                         't','u','v','w','x','y','z');

{*******************************************************************************
* Procedure : Create                                                           *
********************************************************************************
* Purpose   : Inits the instance                                               *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
constructor TBase64.Create;
begin
     Init;
end; {TBase64.Create}

{*******************************************************************************
* Procedure : StringToBase64                                                   *
********************************************************************************
* Purpose   : Encodes the input string into an armoured version                *
********************************************************************************
* Paramters : StringToEncode - the string to be armoured                       *
********************************************************************************
* Returns   : Result - the armoured string                                     *
*******************************************************************************}
function TBase64.StringToBase64(StringToEncode: string): string;
var
     i: integer;
begin
     Init;
     for i := 1 to Length(StringToEncode) do
     begin
          ByteToBase64(Ord(StringToEncode[i]));
     end;
     Base64Final;
     Result := ReturnString;
end;

function TBase64.Base64ToString(StringToDecode: string): string;
var
     i: integer;
begin
     Init;
     for i := 1 to Length(StringToDecode) do
     begin
          Base64ToByte(Ord(StringToDecode[i]));
     end;
     ByteFinal;
     Result := ReturnString;
end;

procedure TBase64.Init;
begin
     Counter := 0;
     CharCounter := 0;
     This := 0;
     Last := 0;
     ReturnString := '';
end;

procedure TBase64.ByteToBase64(b: byte);
begin
     case Counter of
     0:
          begin
               ReturnString := ReturnString + BinToAsc[(b shr 2)];
               Inc(CharCounter);
               if CharCounter = LINELENGTH then
               begin
                    ReturnString := ReturnString + Chr(10) + Chr(13);
                    CharCounter := 0;
               end;
               last := b;
          end;
     1:
          begin
               ReturnString := ReturnString + BinToAsc[((last and $3) shl 4) or ((b and $F0) shr 4) ];
               Inc(CharCounter);
               if CharCounter = LINELENGTH then
               begin
                    ReturnString := ReturnString + Chr(10) + Chr(13);
                    CharCounter := 0;
               end;
               last := b;
          end;
     2:
          begin
               ReturnString := ReturnString + BinToAsc[((last and $F) shl 2) or ((b and $C0) shr 6)];
               Inc(CharCounter);
               if CharCounter = LINELENGTH then
               begin
                    ReturnString := ReturnString + Chr(10) + Chr(13);
                    CharCounter := 0;
               end;
               ReturnString := ReturnString + BinToAsc[(b and $3F)];
               Inc(CharCounter);
               if CharCounter = LINELENGTH then
               begin
                    ReturnString := ReturnString + Chr(10) + Chr(13);
                    CharCounter := 0;
               end;
          end;
     end;
     Inc(Counter);
     if Counter = 3 then
     begin
          Counter := 0;
     end;
end;

function  DecodeBase64(b: byte): byte;
begin
     if (b >= Ord('0')) and (b <= Ord('9')) then
     begin
          Result := b - Ord('0') + 2;
          Exit;
     end;
     if (b >= Ord('A')) and (b <= Ord('Z')) then
     begin
          Result := b - Ord('A') + 12;
          Exit;
     end;
     if (b >= Ord('a')) and (b <= Ord('z')) then
     begin
          Result := b - Ord('a') + 38;
          Exit;
     end;
     if b = Ord('+') then
     begin
          Result := 0;
          Exit;
     end;
     if b = Ord('-') then
     begin
          Result := 1;
          Exit;
     end;
     // Default result - should never get here
     Result := 0;
end;

procedure TBase64.ByteFinal;
begin
     if last <> 0 then
     begin
          Base64ToHex(Ord('0'));
     end;
end;

procedure TBase64.Base64ToHex(c: byte);
begin
     case Counter of
     0:
          begin
               last := DecodeBase64(Ord(c)) shl 2;
          end;
     1:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + IntToHex((last or (this shr 4)) and $ff,2);
               last := this shl 4;
          end;
     2:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + IntToHex((last or (this shr 2)) and $ff,2);
               last := this shl 6;
          end;
     3:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + IntToHex((last or this) and $ff,2);
               last := 0;
          end;
     end;
     Inc(Counter);
     if Counter = 4 then
     begin
          Counter := 0;
     end;
end;

procedure TBase64.Base64ToByte(c: byte);
begin
     // strip out linefeeds / CRs
     if (c = 10) or (c = 13) then
     begin
          Exit
     end;

     case Counter of
     0:
          begin
               last := DecodeBase64(Ord(c)) shl 2;
          end;
     1:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + Chr((last or (this shr 4)) and $ff);
               last := this shl 4;
          end;
     2:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + Chr((last or (this shr 2)) and $ff);
               last := this shl 6;
          end;
     3:
          begin
               this := DecodeBase64(Ord(c));
               ReturnString := ReturnString + Chr((last or this) and $ff);
               last := 0;
          end;
     end;
     Inc(Counter);
     if Counter = 4 then
     begin
          Counter := 0;
     end;
end;

procedure TBase64.Base64Final;
begin
     if Counter <> 0 then
     begin
          ByteToBase64(0);
          If ReturnString[Length(ReturnString)] = '0' then
          begin
               ReturnString := Copy(ReturnString, 0, Length(ReturnString) -1);
          end;
     end;
end;

function  HexToBase64(s: string): string;
var
     i: integer;
     nibble : byte;
     ByteValue: byte;
     Base64: TBase64;
begin
     // Converts a hex string into its base 64 equivalent
     Result := '';
     i := 1;
     Base64 := TBase64.Create;
     While i <= length(s) do
     begin
          nibble := Ord(s[i]) - Ord('0');
          if nibble > 9 then
          begin
               nibble := nibble - 7;
          end;
          ByteValue := nibble;
          Inc(i);
          nibble := Ord(s[i]) - Ord('0');
          if nibble > 9 then
          begin
               nibble := nibble - 7;
          end;
          ByteValue := ByteValue * 16 + nibble;
          Inc(i);

          // convert the byte
          Base64.ByteToBase64(ByteValue);
      end;
end;

function  Base64ToHex(s: string): string;
var
     i : integer;
     Base64: TBase64;
begin
     // a base 64 string to its hex equivalent
     Base64 := TBase64.Create;
     Result := '';
     for i:= 1 to length(s) do
     begin
          Base64.Base64ToHex(Ord(s[i]));
     end;
end;

end.
