{*******************************************************************************
* Unit      : Secret                                                           *
********************************************************************************
* Purpose   : Gets a new secret password from the user                         *
********************************************************************************
* Copyright : This module is copyright TSM Inc. (Ian Sparkes) 1998.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 23.02.98  - 1.0 Original module                                  *
********************************************************************************
* Switches  : none                                                             *
*******************************************************************************}
unit Secret;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

{*******************************************************************************
* Type      : frmNewSecret                                                     *
********************************************************************************
* Purpose   : Defines the form object for the window                           *
*             NOTE: Value of the secret is read by the calling module.         *
*******************************************************************************}
type
  TfrmNewSecret = class(TForm)
    btnOK: TButton;
    grpNewNym: TGroupBox;
    edtSecretConfirm: TEdit;
    lblConfPassword: TLabel;
    edtSecret: TEdit;
    lblPassword: TLabel;
    edtName: TEdit;
    lblKeyOwner: TLabel;
    Label1: TLabel;
    btnCancel: TButton;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure edtNameChange(Sender: TObject);
  end;

var
  frmNewSecret: TfrmNewSecret;

implementation

{$R *.DFM}

{*******************************************************************************
* Procedure : TfrmNewSecret.btnOKClick                                         *
********************************************************************************
* Purpose   : Checks that the password and the confirm are OK and closes       *
********************************************************************************
* Paramters : none                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmNewSecret.btnOKClick(Sender: TObject);
begin
     if edtSecret.Text = edtSecretConfirm.Text then
     begin
          Close;
     end
     else
     begin
          edtSecret.Text := '';
          edtSecretConfirm.Text := '';
     end; {if}
end; {TfrmNewSecret.btnOKClick}

{*******************************************************************************
* Procedure : TfrmNewSecret.btnCancelClick                                     *
********************************************************************************
* Purpose   : Clears the password and closes the window                        *
********************************************************************************
* Paramters : none                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmNewSecret.btnCancelClick(Sender: TObject);
begin
     edtName.Text := '';
     Close;
end; {TfrmNewSecret.btnCancelClick}

{*******************************************************************************
* Procedure : TfrmNewSecret.edtNameChange                                      *
********************************************************************************
* Purpose   : Enables or disables the OK button                                *
********************************************************************************
* Paramters : none                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmNewSecret.edtNameChange(Sender: TObject);
begin
     btnOK.Enabled := (Length(edtName.Text) > 0) and (Length(edtSecret.Text) > 0) and (Length(edtSecretConfirm.Text) > 0);
end; {TfrmNewSecret.edtNameChange}

end.
