{*******************************************************************************
* Unit      : Smail_Main                                                       *
********************************************************************************
* Purpose   : Handles the interface to user                                    *
********************************************************************************
* Copyright : This module is copyright TSM Inc. (Ian Sparkes) 1998.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 28.02.98  - 1.0 Original module                                  *
********************************************************************************
* Switches  : none                                                             *
*******************************************************************************}
unit Smail_main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blowfish, ECCrypt, mdMsgCmp, mdTray, Menus;

type
  TfrmMain = class(TForm)
    btnEncrypt: TButton;
    btnDecrypt: TButton;
    Memo1: TMemo;
    Blowfish1: TBlowfish;
    EllipticCurve1: TEllipticCurve;
    btnNewKey: TButton;
    cboSender: TComboBox;
    lblSender: TLabel;
    lblRecipient: TLabel;
    btnDone: TButton;
    Popup1: TPopupMenu;
    Show1: TMenuItem;
    mdTray1: TmdTray;
    EncryptCli1: TMenuItem;
    N1: TMenuItem;
    lstRecipient: TListBox;
    Exit1: TMenuItem;
    N2: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure btnNewKeyClick(Sender: TObject);
    procedure btnDoneClick(Sender: TObject);
    procedure Show1Click(Sender: TObject);
    procedure lstRecipientClick(Sender: TObject);
    procedure Encrypt;
    procedure Decrypt;
    procedure btnEncryptClick(Sender: TObject);
    procedure btnDecryptClick(Sender: TObject);
    procedure EncryptCli1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
  end;

var
  frmMain: TfrmMain;

implementation

uses
     Armour, Secret, GetSec, KeyDB;

const
     MESSAGESTART     = '--- Begin Message Block ---';
     MESSAGEEND       = '---  End Message Block  ---';
     INFOEND          = '---End Information Block---';
     FILLER           = '     ';
     MSG_NEW_PUB_KEY  = 'This is a public key which you do not possess. Do you want to store it for later use?';
     MSG_NO_SEC_KEY   = 'You do not have a secret key for the message recipient. You cannot decrypt this message';
     MSG_WRONG_SECRET = 'You did not enter the correct secret. The message has not been decrypted.';
     MSG_NO_SECRET    = 'You did not enter a secret. The message has not been decrypted.';
     SYM_KEY_LENGTH   = 32; // the length of the symetric random key

{$R *.DFM}

{*******************************************************************************
* Procedure : Encrypt                                                          *
********************************************************************************
* Purpose   : Encrypts the contents of the buffer using the selected PKs       *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Encrypt;
var
     i: integer;                // general purpose counter
     RecipNumber: integer;      // counter for multiple recipients
     TempString: string;        // general purpose helper string
     TempString1: string;       // general purpose helper string
     RandomNum: string;         // the random symmetric key
     RandomOffset: string;      // the difference between the symmetric key and
                                // session key for a PK
     B64 : TBase64;             // Object instance which handles the armour
     Key: TKeyRecord;           // general purpose key variable
begin
     // create the random number
     // this must be expanded to provide crypto strong RNs - this is reason we
     // should not use SMail for real
     RandomNum := '';
     Randomize;
     for i := 1 to SYM_KEY_LENGTH do
     begin
          RandomNum := RandomNum + Chr(Random(256));
     end; {for}

     // prepare the cipher using the random key
     for i := 0 to Memo1.Lines.Count-1 do
     begin
          TempString := TempString + Memo1.Lines.Strings[i] + Chr(10) + Chr(13);
     end; {for}

     // the blowfish component must be used in ECB mode, unless we want to derive
     // the IV from the symmetric key (e.g. use the first x bytes as the IV)
     Blowfish1.InitialiseString(RandomNum);
     Blowfish1.EncryptString(TempString,TempString1);
     Blowfish1.Burn;

     // remove the plaintext from the buffer
     Memo1.Lines.Clear;

     B64 := TBase64.Create;

     // add the armour to the encrypted text
     TempString := B64.StringToBase64(TempString1);

     // add the formatting to the buffer
     Memo1.Lines.Add(FILLER + MESSAGESTART + FILLER);
     While Length(Tempstring) > 0 do
     begin
          Memo1.Lines.Add(Copy(TempString,0,64));
          TempString := Copy(TempString,67, Length(TempString)-66);
     end;
     // close the ciphertext formatted block
     Memo1.Lines.Add(FILLER + MESSAGEEND + FILLER);

     // **** create the info block ****

     // add the name and PK of the sender
     Memo1.Lines.Add('SE: ' + cboSender.Text);
     GetKey(cboSender.Text, Key);
     Memo1.Lines.Add('PK: ' + Key.PubKey);

     // add the names, Exchanged values and offset keys of the recipients
     for RecipNumber := 0 to lstRecipient.Items.Count-1 do
     begin
          if lstRecipient.Selected[RecipNumber] then
          begin
               // add the recipient to the list
               GetKey(lstRecipient.Items.Strings[RecipNumber],Key);
               Memo1.Lines.Add('RE: ' + Key.Name);

               // create the exchanged value and the offset
               EllipticCurve1.KeyPublic := Key.PubKey;
               EllipticCurve1.Encrypt;
               Memo1.Lines.Add('EV: ' + EllipticCurve1.ExchangedValue);
               RandomOffset := B64.Base64ToString(EllipticCurve1.KeySession);

               SetLength(RandomOffset,SYM_KEY_LENGTH);

               // the offset is the session key delivered by TEllipticCurve
               // xor'ed with the symmetric key. This is necessary as we have
               // multiple recipients, but they must all know the same symmetric key
               for i := 1 to SYM_KEY_LENGTH do
               begin
                    RandomOffset[i] := Chr(Ord(RandomOffset[i]) xor Ord(RandomNum[i]));
               end;

               // add the armour to the offset
               TempString := B64.StringToBase64(RandomOffset);
               Memo1.Lines.Add('OS: ' + TempString);
          end;
     end;

     // close the formatted info block
     Memo1.Lines.Add(FILLER + INFOEND + FILLER);

     // free the armour object
     B64.Free;
end; {TfrmMain.Encrypt}

{*******************************************************************************
* Procedure : Decrypt                                                          *
********************************************************************************
* Purpose   : Decrypts the contents of the buffer using the selected PKs       *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Decrypt;
var
     i: integer;                // general purpose counter
     j: integer;                // general purpose counter
     MessStartIdx: integer;     // the start of the formatted block
     MessEndIdx: integer;       // the end of the encrypted message block
     MessInfoIdx: integer;      // end of the formatted block
     SenderName: string;        // the name of the sender
     PublicKey: string;         // the public key of the sender
     Recipient: string;         // the name of the recipient
     ExchValue: string;         // the exchanged value of the recipient
     TempString: string;        // general purpose helper string
     TempString1: string;       // general purpose helper string
     RandomOffset: string;      // the difference between the symmetric key and
                                // session key for the public key
     Key: TKeyRecord;           // general purpose key record
     B64 : TBase64;             // the instance of the armour object
     frmGetSec: TfrmGetSecret;  // the instance of the form to get the secret
     FoundKey: Boolean;         // marker if we have found a recipient for which
                                // we have a PK
begin
     // make sure the important variables are initialised
     MessStartIdx := 0;
     MessEndIdx := 0;
     MessInfoIdx := 0;
     SenderName := '';
     Recipient := '';
     PublicKey := '';
     ExchValue := '';
     FoundKey := False;

     // search for the start and end markers of the formatted block
     // gather the format info out of the block
     for i := 0 to Memo1.Lines.Count-1 do
     begin
          if Pos(MESSAGESTART,Memo1.Lines.Strings[i]) <> 0 then
          begin
               MessStartIdx := i;
          end; {if}

          if Pos(MESSAGEEND,Memo1.Lines.Strings[i]) <> 0 then
          begin
               MessEndIdx := i;
          end; {if}

          if Pos(INFOEND,Memo1.Lines.Strings[i]) <> 0 then
          begin
               MessInfoIdx := i;
          end; {if}
     end; {for}

     // check the format of the block - if they are all 0, then we have not
     // found a formatted block
     if MessStartIdx + MessEndIdx + MessInfoIdx = 0 then
     begin
          // doesn't appear to be a valid block
          ShowMessage('Not a valid cipher block');
          Exit;
     end; {if}

     // check to see if we know any of the recipients' secrets
     for i := MessEndIdx+1 to MessInfoIdx-1 do
     begin
          if (Pos('RE: ',Memo1.Lines.Strings[i]) <> 0) then
          begin
               Recipient := Copy(Memo1.Lines.Strings[i],Pos('RE: ',Memo1.Lines.Strings[i])+4, Length(Memo1.Lines.Strings[i])-4);
               if GetKey(Recipient,Key) then
               begin
                    // we have found a key, do we have the secret key?
                    if Key.SecKey then
                    begin
                         // we have the secret key, store the stuff
                         j := i + 1;
                         ExchValue := Copy(Memo1.Lines.Strings[j],Pos('EV: ',Memo1.Lines.Strings[j])+4, Length(Memo1.Lines.Strings[j])-4);
                         Inc(j);
                         RandomOffset := Copy(Memo1.Lines.Strings[j],Pos('OS: ',Memo1.Lines.Strings[j])+4, Length(Memo1.Lines.Strings[j])-4);
                         FoundKey := True;
                         Break;
                    end; {if}
               end; {if}
          end; {if}
     end; {for}

     // check to see if we have any new public Keys
     for i := MessEndIdx+1 to MessInfoIdx-1 do
     begin
          if (Pos('SE: ',Memo1.Lines.Strings[i]) <> 0) then
          begin
               SenderName := Copy(Memo1.Lines.Strings[i],Pos('SE: ',Memo1.Lines.Strings[i])+4, Length(Memo1.Lines.Strings[i])-4);
          end; {if}

          if (MessEndIdx > 0) and (Pos('PK: ',Memo1.Lines.Strings[i]) <> 0) then
          begin
               PublicKey := Copy(Memo1.Lines.Strings[i],Pos('PK: ',Memo1.Lines.Strings[i])+4, Length(Memo1.Lines.Strings[i])-4);
          end; {if}
     end; {for}

     // Store the public Key for later use
     if not GetKey(SenderName,Key) then
     begin
          if MessageDlg(MSG_NEW_PUB_KEY ,mtConfirmation,[mbYes, mbNo],0) = mrYes then
          begin
               Key.Name := SenderName;
               Key.PubKey := PublicKey;
               Key.SecKey := False;
               SaveKey(Key);
          end; {if}
     end; {if}

     // check to see if we have the secret key for the message
     if not FoundKey then
     begin
          // if not, then we can't do anything with the information
          MessageDlg(MSG_NO_SEC_KEY , mtError,[mbOK],0);
          Exit;
     end; {if}

     // Get the secret and verify it
     frmGetSec := TfrmGetSecret.Create(self);
     frmGetSec.lblName.Caption := Recipient;
     frmGetSec.ShowModal;

     if Length(frmGetSec.edtSecret.Text) > 0 then
     begin
          // we have a key, now verify it
          // we add in the recipient name here to help avoid a dictionary attack
          // note: we compare the public keys, not the private ones
          GetKey(Recipient,Key);
          EllipticCurve1.PassPhrase := Recipient + frmGetSec.edtSecret.Text;
          EllipticCurve1.MakeKey;
          if not (EllipticCurve1.KeyPublic = Key.PubKey) then
          begin
               // the secret was not correct
               MessageDlg(MSG_WRONG_SECRET, mtError,[mbOK],0);
               Exit;
          end; {if}
     end
     else
     begin
          MessageDlg(MSG_NO_SECRET, mtError,[mbOK],0);
          Exit;
     end; {if}

     // we have finished with the get secret window
     frmGetSec.Free;

     // recover the session key
     EllipticCurve1.ExchangedValue := ExchValue;
     EllipticCurve1.Decrypt;

     // Extract the symmetric key
     B64 := TBase64.Create;
     TempString := B64.Base64ToString(EllipticCurve1.KeySession);
     TempString1 := B64.Base64ToString(RandomOffset);
     SetLength(TempString1,SYM_KEY_LENGTH);
     SetLength(RandomOffset,SYM_KEY_LENGTH);
     for i := 1 to SYM_KEY_LENGTH do
     begin
          RandomOffset[i] := Chr(Ord(TempString[i]) xor Ord(TempString1[i]));
     end; {for}

     // recover the encrypted text
     TempString := '';
     For i := MessStartIdx+1 to MessEndIdx-1 do
     begin
          TempString := TempString + Memo1.Lines.Strings[i];
     end; {for}

     // Remove the base 64 armour
     TempString := B64.Base64ToString(TempString);

     // throw the armour object away
     B64.Free;

     // the blowfish component must be used in ECB mode, unless we want to derive
     // the IV from the symmetric key (e.g. use the first x bytes as the IV)
     Blowfish1.InitialiseString(RandomOffset);
     Blowfish1.DecryptString(TempString,TempString1);
     Blowfish1.Burn;

     TempString := TempString1;

     Memo1.Lines.Clear;

     while Length(Tempstring) > 0 do
     begin
          i := Pos(Chr(10), TempString);
          Memo1.Lines.Add(Copy(TempString,0,i-1));
          TempString := Copy(TempString,i+2, Length(TempString)-i-1);
     end; {while}
end; {TfrmMain.Decrypt}

{*******************************************************************************
* Procedure : TfrmMain.FormCreate                                              *
********************************************************************************
* Purpose   : Loads the key lists from the key file                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
     // load the key lists
     GetKeyList(lstRecipient.Items, False);
     GetKeyList(cboSender.Items, True);
end; {TfrmMain.FormCreate}

{*******************************************************************************
* Procedure : TfrmMain.btnNewKeyClick                                          *
********************************************************************************
* Purpose   : Creates a new key pair                                           *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnNewKeyClick(Sender: TObject);
var
     frmSecret: TfrmNewSecret;
     Keys: TKeyRecord;
begin
     // instantiate the get secret form
     frmSecret := TfrmNewSecret.Create(self);
     frmSecret.ShowModal;
     if Length(frmSecret.edtName.Text) > 0  then
     begin
          // Recipient name in the password to help avoid a dictionary attack
          EllipticCurve1.PassPhrase := frmSecret.edtName.Text + frmSecret.edtSecret.Text;
          EllipticCurve1.MakeKey;
          Keys.Name := frmSecret.edtName.Text;
          Keys.PubKey := EllipticCurve1.KeyPublic;
          Keys.SecKey := True;
          SaveKey(Keys);
     end;

     // we have finished with the get secret window
     frmSecret.Free;

     // re-read the comboboxes
     lstRecipient.Items.Clear;
     cboSender.Items.Clear;
     GetKeyList(lstRecipient.Items, False);
     GetKeyList(cboSender.Items, True);
end; {TfrmMain.btnNewKeyClick}

{*******************************************************************************
* Procedure : TfrmMain.btnDoneClick                                            *
********************************************************************************
* Purpose   : Closes (but does not destroy) the main window                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnDoneClick(Sender: TObject);
begin
     // copy the result of the en- decryption to the clipboard
     Memo1.SelectAll;
     Memo1.CopyToClipboard;

     // hide the main window
     Hide;
end; {TfrmMain.btnDoneClick}

{*******************************************************************************
* Procedure : TfrmMain.Show1Click                                              *
********************************************************************************
* Purpose   : Shows the the main window                                        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Show1Click(Sender: TObject);
begin
     // show the main window
     Show;
end; {TfrmMain.Show1Click}

{*******************************************************************************
* Procedure : TfrmMain.lstRecipientClick                                       *
********************************************************************************
* Purpose   : Enables or disables the 'Encrypt' button                         *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.lstRecipientClick(Sender: TObject);
begin
     // enable the encrypt button when the recipient and the sender have been entered
     frmMain.btnEncrypt.Enabled := ((Length(cboSender.Text) > 0) and
                                    (lstRecipient.SelCount > 0));
end; {TfrmMain.lstRecipientClick}

{*******************************************************************************
* Procedure : TfrmMain.btnEncryptClick                                         *
********************************************************************************
* Purpose   : Services the 'Encrypt' button                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnEncryptClick(Sender: TObject);
begin
     // encrypt the contents of the buffer
     Encrypt;
end; {TfrmMain.btnEncryptClick}

{*******************************************************************************
* Procedure : TfrmMain.btnDecryptClick                                         *
********************************************************************************
* Purpose   : Services the 'Decrypt' button                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnDecryptClick(Sender: TObject);
begin
     // decrypt the contents of the buffer
     Decrypt;
end; {TfrmMain.btnDecryptClick}

{*******************************************************************************
* Procedure : TfrmMain.EncryptCli1Click                                        *
********************************************************************************
* Purpose   : Services the 'Do Clipboard' popup menu point                     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.EncryptCli1Click(Sender: TObject);
begin
     // copy the clipboard into the buffer
     Memo1.Clear;
     Memo1.PasteFromClipboard;

     // show the main form
     Show;
end; {TfrmMain.EncryptCli1Click}

{*******************************************************************************
* Procedure : TfrmMain.Exit1Click                                              *
********************************************************************************
* Purpose   : Exits the program                                                *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Exit1Click(Sender: TObject);
begin
     // exit the program
     Halt;
end; {TfrmMain.Exit1Click}

end.
